package de.jottyfan.minecraft.quickiefabric.container;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.jottyfan.minecraft.quickiefabric.init.RegistryManager;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.inventory.Inventory;
import net.minecraft.item.ItemStack;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.screen.slot.Slot;
import net.minecraft.screen.slot.SlotActionType;
import net.minecraft.util.Hand;

public class BackpackScreenHandler extends ScreenHandler {

	private static final Logger LOGGER = LogManager.getLogger(BackpackScreenHandler.class);

	private final BackpackInventory backpackInventory;
	private final PlayerEntity player;
	private final ItemStack thisStack;
	private final Hand hand;

	public BackpackScreenHandler(int syncId, PlayerInventory playerInventory, PacketByteBuf buf) {
		super(RegistryManager.BACKPACK_SCREEN_HANDLER, syncId);
		this.player = playerInventory.player;
		thisStack = buf.readItemStack();
		this.hand = buf.readByte() > 0 ? Hand.MAIN_HAND : Hand.OFF_HAND;
		this.backpackInventory = BackpackInventory.getInventory(this, player, thisStack);

		backpackInventory.onOpen(player);
		for (int y = 0; y < 6; y++) {
			for (int x = 0; x < 9; ++x) {
				this.addSlot(new Slot(backpackInventory, x + y * 9, 8 + x * 18, -10 + y * 18));
			}
		}
		for (int y = 0; y < 3; ++y) {
			for (int x = 0; x < 9; ++x) {
				this.addSlot(new Slot(playerInventory, x + (y * 9) + 9, 8 + x * 18, 112 + y * 18));
			}
		}
		for (int x = 0; x < 9; ++x) {
			this.addSlot(new Slot(playerInventory, x, 8 + x * 18, 170));
		}
	}

	public PlayerEntity getPlayer() {
		return this.player;
	}

	public Inventory getInventory() {
		return backpackInventory;
	}

	@Override
	public boolean canUse(PlayerEntity playerEntity) {
		return backpackInventory.canPlayerUse(playerEntity);
	}

	@Override
	public void close(PlayerEntity player) {
		this.backpackInventory.setHand(hand);
		this.backpackInventory.onClose(player);
		super.close(player);
	}

	@Override
	public void onSlotClick(int slotId, int quickCraftData, SlotActionType actionType, PlayerEntity playerEntity) {
		Slot slot = slotId >= 0 && slotId < this.slots.size() ? this.slots.get(slotId) : null;
		ItemStack stack = slot == null ? ItemStack.EMPTY : slot.getStack();
		if (actionType.equals(SlotActionType.PICKUP) || actionType.equals(SlotActionType.PICKUP_ALL)
				|| actionType.equals(SlotActionType.SWAP)) {
			super.onSlotClick(slotId, quickCraftData, actionType, playerEntity);
		} else if (actionType.equals(SlotActionType.THROW)) { // pickup from the active items line
			if (stack.getName().equals(thisStack.getName())) {
				return; // omit to put the backpack into itself; as a side effect, omits adding same
								// colored backpacks into this one
			} else {
				// do not throw but pickup
				super.onSlotClick(slotId, quickCraftData, SlotActionType.PICKUP, playerEntity);
			}
		} else if (actionType.equals(SlotActionType.QUICK_CRAFT)) { // drag - but does not drag the right slot content, stack is empty
			if (slotId < 0) {
				return; // do not execute anything if slotId is < 0 as sometimes -999
			}
			if (playerEntity.world.isClient) {
				LOGGER.debug("ignoring action type {} on slotId {} for {} x {}", actionType.toString(), slotId,
						stack.getCount(), stack.getItem().toString());
			}
		} else if (actionType.equals(SlotActionType.QUICK_MOVE)) { // shift click
			if (stack.getName().equals(thisStack.getName())) {
				return; // omit to put the backpack into itself; as a side effect, omits adding same
								// colored backpacks into this one
			} else {
				// do not move but pickup all
				super.onSlotClick(slotId, quickCraftData, SlotActionType.PICKUP_ALL, playerEntity);
			}
		} else {
			if (playerEntity.world.isClient) {
				LOGGER.debug("ignoring action type {} on slotId {} for {} x {}", actionType.toString(), slotId,
						stack.getCount(), stack.getItem().toString());
			}
		}
	}

	@Environment(EnvType.CLIENT)
	public int getRows() {
		return 6;
	}
}
